@@ -11,17 +11,27 @@ import docker
 import redis
 import os
 from pathlib import Path
-from redistimeseries.client import Client
+
+from redisbench_admin.profilers.profilers_local import (
+    check_compatible_system_and_kernel_and_prepare_profile,
+    profilers_start_if_required,
+    local_profilers_platform_checks,
+    profilers_stop_if_required,
+)
 from docker.models.containers import Container
 
 from redisbench_admin.run.common import (
     get_start_time_vars,
     prepare_benchmark_parameters,
 )
+from redisbench_admin.run.grafana import generate_artifacts_table_grafana_redis
 from redisbench_admin.utils.benchmark_config import (
     get_final_benchmark_config,
 )
-from redisbench_admin.run.redistimeseries import timeseries_test_sucess_flow
+from redisbench_admin.run.redistimeseries import (
+    timeseries_test_sucess_flow,
+    datasink_profile_tabular_data,
+)
 from redisbench_admin.run.run import calculate_client_tool_duration_and_check
 from redisbench_admin.utils.benchmark_config import (
     extract_redis_dbconfig_parameters,
@@ -37,6 +47,7 @@ from redis_benchmarks_specification.__common__.env import (
     STREAM_GH_NEW_BUILD_RUNNERS_CG,
     REDIS_HEALTH_CHECK_INTERVAL,
     REDIS_SOCKET_TIMEOUT,
+    S3_BUCKET_NAME,
 )
 from redis_benchmarks_specification.__common__.package import (
     get_version_string,
@@ -126,7 +137,7 @@ def main():
         )
         logging.error("Error message {}".format(e.__str__()))
         exit(1)
-    rts = None
+    datasink_conn = None
     if args.datasink_push_results_redistimeseries:
         logging.info(
             "Checking redistimeseries datasink connection is available at: {}:{} to push the timeseries data".format(
@@ -134,7 +145,7 @@ def main():
             )
         )
         try:
-            rts = Client(
+            datasink_conn = redis.StrictRedis(
                 host=args.datasink_redistimeseries_host,
                 port=args.datasink_redistimeseries_port,
                 decode_responses=True,
@@ -144,7 +155,7 @@ def main():
                 socket_connect_timeout=REDIS_SOCKET_TIMEOUT,
                 socket_keepalive=True,
             )
-            rts.redis.ping()
+            datasink_conn.ping()
         except redis.exceptions.ConnectionError as e:
             logging.error(
                 "Unable to connect to redis available at: {}:{}".format(
@@ -161,9 +172,27 @@ def main():
     stream_id = None
     docker_client = docker.from_env()
     home = str(Path.home())
+    cpuset_start_pos = args.cpuset_start_pos
+    logging.info("Start CPU pinning at position {}".format(cpuset_start_pos))
+
     # TODO: confirm we do have enough cores to run the spec
     # availabe_cpus = args.cpu_count
     datasink_push_results_redistimeseries = args.datasink_push_results_redistimeseries
+    grafana_profile_dashboard = args.grafana_profile_dashboard
+
+    profilers_list = []
+    profilers_enabled = args.enable_profilers
+    if profilers_enabled:
+        profilers_list = args.profilers.split(",")
+        res = check_compatible_system_and_kernel_and_prepare_profile(args)
+        if res is False:
+            logging.error(
+                "Requested for the following profilers to be enabled but something went wrong: {}.".format(
+                    " ".join(profilers_list)
+                )
+            )
+            exit(1)
+
     logging.info("Entering blocking read waiting for work.")
     if stream_id is None:
         stream_id = args.consumer_start_id
@@ -174,10 +203,14 @@ def main():
             docker_client,
             home,
             stream_id,
-            rts,
+            datasink_conn,
             testsuite_spec_files,
             topologies_map,
             running_platform,
+            profilers_enabled,
+            profilers_list,
+            grafana_profile_dashboard,
+            cpuset_start_pos,
         )
 
 
@@ -215,10 +248,14 @@ def self_contained_coordinator_blocking_read(
     docker_client,
     home,
     stream_id,
-    rts,
+    datasink_conn,
     testsuite_spec_files,
     topologies_map,
     platform_name,
+    profilers_enabled,
+    profilers_list,
+    grafana_profile_dashboard="",
+    cpuset_start_pos=0,
 ):
     num_process_streams = 0
     num_process_test_suites = 0
@@ -246,10 +283,14 @@ def self_contained_coordinator_blocking_read(
             docker_client,
             home,
             newTestInfo,
-            rts,
+            datasink_conn,
             testsuite_spec_files,
             topologies_map,
             platform_name,
+            profilers_enabled,
+            profilers_list,
+            grafana_profile_dashboard,
+            cpuset_start_pos,
         )
         num_process_streams = num_process_streams + 1
         num_process_test_suites = num_process_test_suites + total_test_suite_runs
@@ -308,10 +349,14 @@ def process_self_contained_coordinator_stream(
     docker_client,
     home,
     newTestInfo,
-    rts,
+    datasink_conn,
     testsuite_spec_files,
     topologies_map,
     running_platform,
+    profilers_enabled=False,
+    profilers_list=[],
+    grafana_profile_dashboard="",
+    cpuset_start_pos=0,
 ):
     stream_id = "n/a"
     overall_result = False
@@ -335,6 +380,8 @@ def process_self_contained_coordinator_stream(
             ) = extract_build_info_from_streamdata(testDetails)
 
             overall_result = True
+            profiler_dashboard_links = []
+
             for test_file in testsuite_spec_files:
                 redis_containers = []
                 client_containers = []
@@ -376,7 +423,7 @@ def process_self_contained_coordinator_stream(
                     for topology_spec_name in benchmark_config["redis-topologies"]:
                         test_result = False
                         try:
-                            current_cpu_pos = 0
+                            current_cpu_pos = cpuset_start_pos
                             ceil_db_cpu_limit = extract_db_cpu_limit(
                                 topologies_map, topology_spec_name
                             )
@@ -389,7 +436,30 @@ def process_self_contained_coordinator_stream(
                             )
                             tf_github_org = "redis"
                             tf_github_repo = "redis"
+                            setup_name = "oss-standalone"
                             tf_triggering_env = "ci"
+                            github_actor = "{}-{}".format(
+                                tf_triggering_env, running_platform
+                            )
+                            dso = "redis-server"
+                            profilers_artifacts_matrix = []
+
+                            collection_summary_str = ""
+                            if profilers_enabled:
+                                collection_summary_str = (
+                                    local_profilers_platform_checks(
+                                        dso,
+                                        github_actor,
+                                        git_branch,
+                                        tf_github_repo,
+                                        git_hash,
+                                    )
+                                )
+                                logging.info(
+                                    "Using the following collection summary string for profiler description: {}".format(
+                                        collection_summary_str
+                                    )
+                                )
 
                             restore_build_artifacts_from_test_details(
                                 build_artifacts, conn, temporary_dir, testDetails
@@ -423,10 +493,15 @@ def process_self_contained_coordinator_stream(
                                 network_mode="host",
                                 detach=True,
                                 cpuset_cpus=db_cpuset_cpus,
+                                pid_mode="host",
                             )
                             redis_containers.append(container)
+
                             r = redis.StrictRedis(port=6379)
                             r.ping()
+                            redis_pids = []
+                            first_redis_pid = r.info()["process_id"]
+                            redis_pids.append(first_redis_pid)
                             ceil_client_cpu_limit = extract_client_cpu_limit(
                                 benchmark_config
                             )
@@ -506,6 +581,23 @@ def process_self_contained_coordinator_stream(
                             client_container_image = extract_client_container_image(
                                 benchmark_config
                             )
+                            profiler_call_graph_mode = "dwarf"
+                            profiler_frequency = 99
+                            # start the profile
+                            (
+                                profiler_name,
+                                profilers_map,
+                            ) = profilers_start_if_required(
+                                profilers_enabled,
+                                profilers_list,
+                                redis_pids,
+                                setup_name,
+                                start_time_str,
+                                test_name,
+                                profiler_frequency,
+                                profiler_call_graph_mode,
+                            )
+
                             logging.info(
                                 "Using docker image {} as benchmark client image (cpuset={}) with the following args: {}".format(
                                     client_container_image,
@@ -541,6 +633,79 @@ def process_self_contained_coordinator_stream(
                             )
                             logging.info("output {}".format(client_container_stdout))
                             r.shutdown(save=False)
+
+                            (_, overall_tabular_data_map,) = profilers_stop_if_required(
+                                datasink_push_results_redistimeseries,
+                                benchmark_duration_seconds,
+                                collection_summary_str,
+                                dso,
+                                tf_github_org,
+                                tf_github_repo,
+                                profiler_name,
+                                profilers_artifacts_matrix,
+                                profilers_enabled,
+                                profilers_map,
+                                redis_pids,
+                                S3_BUCKET_NAME,
+                                test_name,
+                            )
+                            if (
+                                profilers_enabled
+                                and datasink_push_results_redistimeseries
+                            ):
+                                datasink_profile_tabular_data(
+                                    git_branch,
+                                    tf_github_org,
+                                    tf_github_repo,
+                                    git_hash,
+                                    overall_tabular_data_map,
+                                    conn,
+                                    setup_name,
+                                    start_time_ms,
+                                    start_time_str,
+                                    test_name,
+                                    tf_triggering_env,
+                                )
+                                if len(profilers_artifacts_matrix) == 0:
+                                    logging.error("No profiler artifact was retrieved")
+                                else:
+                                    profilers_artifacts = []
+                                    for line in profilers_artifacts_matrix:
+                                        artifact_name = line[2]
+                                        s3_link = line[4]
+                                        profilers_artifacts.append(
+                                            {
+                                                "artifact_name": artifact_name,
+                                                "s3_link": s3_link,
+                                            }
+                                        )
+                                    https_link = generate_artifacts_table_grafana_redis(
+                                        datasink_push_results_redistimeseries,
+                                        grafana_profile_dashboard,
+                                        profilers_artifacts,
+                                        datasink_conn,
+                                        setup_name,
+                                        start_time_ms,
+                                        start_time_str,
+                                        test_name,
+                                        tf_github_org,
+                                        tf_github_repo,
+                                        git_hash,
+                                        git_branch,
+                                    )
+                                    profiler_dashboard_links.append(
+                                        [
+                                            setup_name,
+                                            test_name,
+                                            " {} ".format(https_link),
+                                        ]
+                                    )
+                                    logging.info(
+                                        "Published new profile info for this testcase. Access it via: {}".format(
+                                            https_link
+                                        )
+                                    )
+
                             datapoint_time_ms = start_time_ms
                             if (
                                 use_git_timestamp is True
@@ -587,10 +752,10 @@ def process_self_contained_coordinator_stream(
                                 dataset_load_duration_seconds,
                                 None,
                                 topology_spec_name,
-                                "oss-standalone",
+                                setup_name,
                                 None,
                                 results_dict,
-                                rts,
+                                datasink_conn,
                                 datapoint_time_ms,
                                 test_name,
                                 git_branch,
